RequireScript("httpgenericresponse.js");
RequireScript("httpfileresponse.js");
RequireScript("httpjsresponse.js");

function HttpClient(socket, server)
{
	this.socket = socket;

	this.server = server;
	
	this.read_buffer = "";
	
	this.received_headers = false;
	this.response = false;
	this.request = false;
	this.headers = {};
}

HttpClient.prototype.update = function()
{
	if (!this.socket.isConnected())
	{
		this.finished = true;
	}
	
	var read_size;
	
	if (read_size = this.socket.getPendingReadSize())
	{
		this.read_buffer += CreateStringFromByteArray(this.socket.read(read_size));
	}
	
	if (!this.received_headers)
	{
		while (this.read_buffer.indexOf("\r\n") >= 0)
		{
			var header = this.read_buffer.substr(0, this.read_buffer.indexOf("\r\n"));
			//this.server.log.write(header);
			if (header != "")
			{
				if (!this.request)
				{
					this.request = this.split_request(header);
				}
				else
				{
					var h = this.split_header(header);
					this.headers[h[0]] = h[1];
				}
			}
			else
			{
				// end of headers
				this.received_headers = true;
			}
			this.read_buffer = this.read_buffer.substr(this.read_buffer.indexOf("\r\n") + 2);
		}
	}
	
	if (this.received_headers && !this.response)
	{
		if (this.request[0] != "GET")
		{
			this.response = new HttpGenericResponse(501, this);
		}
		else
		{
			// check if this request contains the magic dots
			if (this.request[1].indexOf("..") >= 0)
			{
				this.response = new HttpGenericResponse(400, this);
			}
			else
			{
				// check if the file exists
				var found = this.path_exists(this.request[1]);
				if (this.path_is_directory(this.request[1]))
				{
					found = false;
					// try an index page.
					for (var i = 0; i < this.server.default_index.length; i++)
					{
						var path = this.request[1] + "/" + this.server.default_index[i];
						if (this.path_exists(path))
						{
							found = true;
							this.request[1] = path;
							break;
						}
					}
				}
				var path_to = this.path_split(this.request[1]);
				if (!found)
				{
					this.server.log.write("404 for " + this.request[1] + ".");
					this.response = new HttpGenericResponse(404, this);
				}
				else if (this.request[1].split(".").pop() == "jhtml")
				{
					this.server.log.write("Jhtml file " + this.request[1]);
					this.response = new HttpJsResponse("htdocs/" + path_to.join("/"), this);
					this.response.parse();
					if (this.response.replace_with)
					{
						this.server.log.write("Replaced with a new response.");
						this.response = this.response.replace_with;
					}
				}
				else
				{
					this.server.log.write("Found file " + this.request[1]);
					this.response = new HttpFileResponse("htdocs/" + path_to.join("/"), this);
				}
			}
		}
		
		if (this.response)
		{
			// persistent connections are currently causing problems, so they're disabled for now
			this.response.headers['Connection'] = "close";
		}
	}
	
	if (this.response)
	{
		this.response.send_next(this);
		if (this.response.finished)
		{
			this.server.log.write("Finished sending response");
			this.server.log.write("Close mode");
			this.finished = true;
			/* Here's the persistent connection code
				this.server.log.write("Keep-Alive mode");
				this.read_buffer = "";
				this.received_headers = false
				this.request = false;
				this.response = false;
				this.headers = {}; */
		}
	}
}

HttpClient.prototype.split_request = function(req)
{
	var rte = req.indexOf(" ");
	var request_type = req.substring(0, rte);
	var rue = req.indexOf(" ", rte + 1);
	var request_uri = req.substring(rte + 1, rue);
	var http_ver = req.substring(rue + 1);
	return [request_type, request_uri, http_ver];
}

HttpClient.prototype.split_header = function(header)
{
	var hne = header.indexOf(":");
	var header_name = header.substr(0, hne);
	var header_value = header.substr(hne + 1).replace(/^\W*/i, "").replace(/\W$/i, "");
	return [header_name, header_value];
}

HttpClient.prototype.path_exists = function(path)
{
  var path_to = path.split("/");
  
  path_to.remove_all("");
  
  var check_dir = GetFileList("htdocs");
  var found = true;
  for (var i = 0; i < path_to.length; i++)
  {
    if (!check_dir.contains(path_to[i]))
    {
      found = false;
      break;
    }
    else
    {
      check_dir = GetFileList("htdocs/" + path_to.slice(0, i).join("/"));
    }
  }
  return found;
}

HttpClient.prototype.path_is_directory = function(path)
{
  var path_to = path.split("/");
  
  path_to.remove_all("");

  return GetFileList("htdocs/" + path_to.join("/")).length > 0;
}
HttpClient.prototype.path_split = function(path)
{
  var path_to = path.split("/");
  
  path_to.remove_all("");
  return path_to;
}